package org.apache.commons.lang3;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.apache.commons.lang3.CharRange;

public class CharSet implements Serializable {
  private static final long serialVersionUID = 5947847346149275958L;
  
  public static final org.apache.commons.lang3.CharSet EMPTY = new org.apache.commons.lang3.CharSet(new String[] { (String)null });
  
  public static final org.apache.commons.lang3.CharSet ASCII_ALPHA = new org.apache.commons.lang3.CharSet(new String[] { "a-zA-Z" });
  
  public static final org.apache.commons.lang3.CharSet ASCII_ALPHA_LOWER = new org.apache.commons.lang3.CharSet(new String[] { "a-z" });
  
  public static final org.apache.commons.lang3.CharSet ASCII_ALPHA_UPPER = new org.apache.commons.lang3.CharSet(new String[] { "A-Z" });
  
  public static final org.apache.commons.lang3.CharSet ASCII_NUMERIC = new org.apache.commons.lang3.CharSet(new String[] { "0-9" });
  
  protected static final Map<String, org.apache.commons.lang3.CharSet> COMMON = Collections.synchronizedMap(new HashMap<>());
  
  static {
    COMMON.put(null, EMPTY);
    COMMON.put("", EMPTY);
    COMMON.put("a-zA-Z", ASCII_ALPHA);
    COMMON.put("A-Za-z", ASCII_ALPHA);
    COMMON.put("a-z", ASCII_ALPHA_LOWER);
    COMMON.put("A-Z", ASCII_ALPHA_UPPER);
    COMMON.put("0-9", ASCII_NUMERIC);
  }
  
  private final Set<CharRange> set = Collections.synchronizedSet(new HashSet<>());
  
  public static org.apache.commons.lang3.CharSet getInstance(String... setStrs) {
    if (setStrs == null)
      return null; 
    if (setStrs.length == 1) {
      org.apache.commons.lang3.CharSet common = COMMON.get(setStrs[0]);
      if (common != null)
        return common; 
    } 
    return new org.apache.commons.lang3.CharSet(setStrs);
  }
  
  protected CharSet(String... set) {
    for (String s : set)
      add(s); 
  }
  
  protected void add(String str) {
    if (str == null)
      return; 
    int len = str.length();
    int pos = 0;
    while (pos < len) {
      int remainder = len - pos;
      if (remainder >= 4 && str.charAt(pos) == '^' && str.charAt(pos + 2) == '-') {
        this.set.add(CharRange.isNotIn(str.charAt(pos + 1), str.charAt(pos + 3)));
        pos += 4;
        continue;
      } 
      if (remainder >= 3 && str.charAt(pos + 1) == '-') {
        this.set.add(CharRange.isIn(str.charAt(pos), str.charAt(pos + 2)));
        pos += 3;
        continue;
      } 
      if (remainder >= 2 && str.charAt(pos) == '^') {
        this.set.add(CharRange.isNot(str.charAt(pos + 1)));
        pos += 2;
        continue;
      } 
      this.set.add(CharRange.is(str.charAt(pos)));
      pos++;
    } 
  }
  
  CharRange[] getCharRanges() {
    return this.set.<CharRange>toArray(CharRange.EMPTY_ARRAY);
  }
  
  public boolean contains(char ch) {
    synchronized (this.set) {
      for (CharRange range : this.set) {
        if (range.contains(ch))
          return true; 
      } 
    } 
    return false;
  }
  
  public boolean equals(Object obj) {
    if (obj == this)
      return true; 
    if (!(obj instanceof org.apache.commons.lang3.CharSet))
      return false; 
    org.apache.commons.lang3.CharSet other = (org.apache.commons.lang3.CharSet)obj;
    return this.set.equals(other.set);
  }
  
  public int hashCode() {
    return 89 + this.set.hashCode();
  }
  
  public String toString() {
    return this.set.toString();
  }
}
